import * as vscode from "vscode";
import * as child_process from "child_process";

const GitFilter: { [key: string]: string } = {
  s: "since",
  u: "until",
  a: "author",
  g: "grep",
};
export interface GitCommit {
  hash?: string;
  author?: string;
  date?: string;
  msg?: string;
  files?: string[];
}

export class GitService {
  private _repoPath: string;

  constructor(private gitPath: string) {
    const workspaceFolders = vscode.workspace.workspaceFolders;
    if (!workspaceFolders || workspaceFolders.length === 0) {
      throw new Error("No workspace folder opened");
    }
    this._repoPath = workspaceFolders[0].uri.fsPath;
  }
  private defaultFilter(): string[] {
    return ['--author=$(git config user.name)', '--since=\"7 days ago\"'];
  }

  public async getCommits(filter: string = ""): Promise<any[]> {
    const filterParts = filter.split(" ");
    let nFilter: string[] = [];
    filterParts.forEach((part) => {
      const t = part.split(":");
      const key = t[0];
      const v = t[1];
      if (key in GitFilter) {
        nFilter.push(`--${GitFilter[key]}=${v}`);
      }
    });
    if(nFilter.length === 0){
      nFilter = this.defaultFilter();
    }
    const gitArgs = [
      "log",
      "--oneline",
      ...nFilter,
      '--pretty=format:"{\\"hash\\":\\"%h\\",\\"author\\":\\"%an\\",\\"date\\":\\"%ad\\",\\"msg\\":\\"%s\\"}"%n---',
      "--date=format:%Y-%m-%d",
      "--name-only",
    ];
 
    
    return new Promise((resolve, reject) => {
      let lines: string[] = [];
      let commits: GitCommit[] = [];
      function makeCommit(lines: any[]): GitCommit | undefined {
        try {
          if(lines.length === 1) {
            return undefined;
          }
            const commitLine = lines[0];
            console.log(commitLine);
          const commit: GitCommit = JSON.parse(lines[0]);
          commit.files = [];
          for (let i = 1; i < lines.length; i++) {
            if (lines[i].trim() !== "") {
              commit.files?.push(lines[i]);
            }
          }
          return commit;
        } catch (e) {
          console.log(e);
        }

        return undefined;
      }
      const gitProcess = child_process.spawn(this.gitPath, gitArgs, {
        cwd: this._repoPath,
        shell: true,
      });
      gitProcess.stderr.setEncoding("utf8");
      gitProcess.stderr.on("data", (data) => {
        console.log(`stderr: ${data}`);
      });
      gitProcess.stdout.setEncoding("utf8");
      gitProcess.on("error", (error) => {
        console.log(`error: ${error.message}`);
        reject(error);
      });
      gitProcess.stdout.on("data", (data) => {
        const nlines = data.split("\n").filter((l:string) => l.trim() !== "");
        for (let nline of nlines) {
          if (nline.startsWith("{")) {
            if(lines.length > 0){
              const commit = makeCommit(lines);
              if(commit){
                commits.push(commit);
              }
            }
            lines = [];
            lines.push(nline);
          } else if (nline.startsWith("---") ) {
            continue
          }else if( nline.trim() === ""){
            const commit = makeCommit(lines);
            if(commit){
              commits.push(commit);
            }
          } else if (nline.trim() !== "") {
            lines.push(nline);
          }
        }
        if(lines.length > 0){
          const commit = makeCommit(lines);
          if(commit){
            commits.push(commit);
          }
        }
      });
      gitProcess.on("exit",(code,signal)=>{
        if(code === 0){
          resolve(commits);
        }else{
            reject(new Error(`git log exited with code ${code}`));
        }
      });
    });
  }

  public async getPreviousCommit(hash: string): Promise<string> {
    const command = `git log --pretty=format:"%h" -n 1 ${hash}^`;
    return new Promise((resolve, reject) => {
      child_process.exec(
        command,
        { cwd: this._repoPath },
        (error, stdout, stderr) => {
          if (error) {
            reject(error);
          } else {
            resolve(stdout.trim());
          }
        }
      );
    });
  }
}
